"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.List = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const setup_1 = require("@salesforce/lwc-dev-mobile-core/lib/cli/commands/force/lightning/local/setup");
const AndroidUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidUtils");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSUtils");
const PerformanceMarkers_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/PerformanceMarkers");
const chalk_1 = __importDefault(require("chalk"));
const cli_ux_1 = __importDefault(require("cli-ux"));
const perf_hooks_1 = require("perf_hooks");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'device-list');
class List extends setup_1.Setup {
    constructor() {
        super(...arguments);
        this.examples = [
            `sfdx force:lightning:local:device:list -p iOS`,
            `sfdx force:lightning:local:device:list -p Android`
        ];
        this.perfMarker = PerformanceMarkers_1.PerformanceMarkers.getByName(PerformanceMarkers_1.PerformanceMarkers.FETCH_DEVICES_MARKER_KEY);
    }
    async run(direct = false) {
        if (direct) {
            await this.init();
        }
        this.logger.info(`Device List command invoked for ${this.flags.platform}`);
        return this.validateInputParameters()
            .then(() => Common_1.CommandLineUtils.platformFlagIsIOS(this.flags.platform)
            ? this.iOSDeviceList()
            : this.androidDeviceList());
    }
    async iOSDeviceList() {
        CommonUtils_1.CommonUtils.startCliAction('Device List', 'Generating list of supported simulators');
        perf_hooks_1.performance.mark(this.perfMarker.startMarkName);
        const result = await IOSUtils_1.IOSUtils.getSupportedSimulators();
        perf_hooks_1.performance.mark(this.perfMarker.endMarkName);
        CommonUtils_1.CommonUtils.stopCliAction();
        this.showDeviceList(result);
        return Promise.resolve(result);
    }
    async androidDeviceList() {
        CommonUtils_1.CommonUtils.startCliAction('Device List', 'Generating list of supported simulators');
        perf_hooks_1.performance.mark(this.perfMarker.startMarkName);
        const result = await AndroidUtils_1.AndroidUtils.fetchEmulators();
        perf_hooks_1.performance.mark(this.perfMarker.endMarkName);
        CommonUtils_1.CommonUtils.stopCliAction();
        this.showDeviceList(result);
        return Promise.resolve(result);
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:local:device:list', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    showDeviceList(list) {
        let duration = 0;
        const obs = new perf_hooks_1.PerformanceObserver((items, observer) => {
            duration = items.getEntries()[0].duration / 1000;
            perf_hooks_1.performance.clearMarks();
            observer.disconnect();
        });
        obs.observe({ entryTypes: ['measure'] });
        perf_hooks_1.performance.measure(this.perfMarker.name, this.perfMarker.startMarkName, this.perfMarker.endMarkName);
        const message = `DeviceList (${duration.toFixed(3)} sec)`;
        const tree = cli_ux_1.default.tree();
        tree.insert(message);
        const rootNode = tree.nodes[message];
        list.forEach((item) => {
            rootNode.insert(chalk_1.default.bold.green(item));
        });
        tree.display();
    }
}
exports.List = List;
List.description = messages.getMessage('commandDescription');
List.flagsConfig = {
    platform: command_1.flags.string({
        char: 'p',
        description: messages.getMessage('platformFlagDescription'),
        longDescription: messages.getMessage('platformFlagDescription'),
        required: true
    })
};
//# sourceMappingURL=list.js.map