"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Start = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const setup_1 = require("@salesforce/lwc-dev-mobile-core/lib/cli/commands/force/lightning/local/setup");
const AndroidUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidUtils");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSUtils");
const util_1 = __importDefault(require("util"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'device-start');
const LWC_DEV_MOBILE = 'lwc-dev-mobile';
const DEVICE_START = 'Device Start';
class Start extends setup_1.Setup {
    constructor() {
        super(...arguments);
        this.examples = [
            `sfdx force:lightning:local:device:start -p iOS -t MySimulator`,
            `sfdx force:lightning:local:device:create -p Android -t MyEmulator -w`
        ];
        this.platform = '';
        this.target = '';
        this.writableSystem = false;
    }
    async run(direct = false) {
        if (direct) {
            await this.init();
        }
        this.logger.info(`Device Start command invoked for ${this.flags.platform}`);
        return super
            .run(direct)
            .then(() => {
            this.logger.info('Setup requirements met, continuing with Device Start');
            return this.executeDeviceStart();
        });
    }
    async validateInputParameters() {
        return super.validateInputParameters().then(() => {
            const target = this.flags.target;
            const writableSystem = this.flags.writablesystem;
            if (target == null || target.trim() === '') {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidTargetFlagsDescription'), LWC_DEV_MOBILE, this.examples));
            }
            this.platform = this.flags.platform;
            this.target = target;
            this.writableSystem = writableSystem;
            return Promise.resolve();
        });
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:local:device:start', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    async executeDeviceStart() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.platform);
        return isAndroid
            ? this.executeAndroidDeviceStart()
            : this.executeIOSDeviceStart();
    }
    async executeAndroidDeviceStart() {
        return AndroidUtils_1.AndroidUtils.hasEmulator(this.target)
            .then((hasEmulator) => {
            if (!hasEmulator) {
                return Promise.reject(new core_1.SfdxError(util_1.default.format(messages.getMessage('error:targetDoesNotExistDescription'), this.target), LWC_DEV_MOBILE));
            }
            else {
                CommonUtils_1.CommonUtils.startCliAction(DEVICE_START, `Starting device ${this.target}`);
                return AndroidUtils_1.AndroidUtils.startEmulator(this.target, this.writableSystem, false);
            }
        })
            .then((actualPort) => {
            CommonUtils_1.CommonUtils.stopCliAction(`device ${this.target} started on port ${actualPort}, writable system = ${this.writableSystem}`);
            return Promise.resolve();
        });
    }
    async executeIOSDeviceStart() {
        let simDeviceUDID = '';
        return IOSUtils_1.IOSUtils.getSimulator(this.target)
            .then((simDevice) => {
            if (simDevice === null) {
                return Promise.reject(new core_1.SfdxError(util_1.default.format(messages.getMessage('error:targetDoesNotExistDescription'), this.target), LWC_DEV_MOBILE));
            }
            else {
                simDeviceUDID = simDevice.udid;
                return Promise.resolve();
            }
        })
            .then(() => {
            CommonUtils_1.CommonUtils.startCliAction(DEVICE_START, `Starting device ${this.target} (${simDeviceUDID})`);
            return IOSUtils_1.IOSUtils.bootDevice(simDeviceUDID, false);
        })
            .then(() => IOSUtils_1.IOSUtils.launchSimulatorApp())
            .then(() => {
            CommonUtils_1.CommonUtils.stopCliAction(`device ${this.target} (${simDeviceUDID}) started.`);
            return Promise.resolve();
        });
    }
}
exports.Start = Start;
Start.description = messages.getMessage('commandDescription');
Start.flagsConfig = {
    platform: command_1.flags.string({
        char: 'p',
        description: messages.getMessage('platformFlagDescription'),
        required: true
    }),
    target: command_1.flags.string({
        char: 't',
        description: messages.getMessage('targetFlagDescription'),
        required: true
    }),
    writablesystem: command_1.flags.boolean({
        char: 'w',
        description: messages.getMessage('writablesystemFlagDescription'),
        required: false,
        default: false
    })
};
//# sourceMappingURL=start.js.map