"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LwcServerIsRunningRequirement = exports.LWCServerPluginInstalledRequirement = exports.Preview = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const setup_1 = require("@salesforce/lwc-dev-mobile-core/lib/cli/commands/force/lightning/local/setup");
const AndroidLauncher_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidLauncher");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSLauncher_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSLauncher");
const PlatformConfig_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/PlatformConfig");
const PreviewUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/PreviewUtils");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const configSchema = __importStar(require("./previewConfigurationSchema.json"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'preview');
class Preview extends setup_1.Setup {
    constructor() {
        super(...arguments);
        this.examples = [
            `$ sfdx force:lightning:lwc:preview -p iOS -t LWCSim2 -n HellowWordComponent`,
            `$ sfdx force:lightning:lwc:preview -p Android -t LWCEmu2 -n HellowWordComponent`
        ];
        this.serverPort = '';
        this.deviceName = '';
        this.componentName = '';
        this.targetApp = '';
        this.projectDir = '';
        this.configFilePath = '';
    }
    async run(direct = false) {
        if (direct) {
            await this.init();
        }
        this.logger.info(`Preview command invoked for ${this.flags.platform}`);
        return this.validateInputParameters()
            .then(() => {
            if (PreviewUtils_1.PreviewUtils.useLwcServerForPreviewing(this.targetApp, this.appConfig)) {
                const extraReqs = [
                    new LWCServerPluginInstalledRequirement(this.logger),
                    new LwcServerIsRunningRequirement(this.logger)
                ];
                this.addAdditionalRequirements(extraReqs);
            }
            return super.run(direct);
        })
            .then(() => {
            this.logger.info('Setup requirements met, continuing with preview');
            return this.launchPreview();
        })
            .catch((error) => {
            this.logger.warn(`Preview failed for ${this.flags.platform}.`);
            return Promise.reject(error);
        });
    }
    async validateInputParameters() {
        return super.validateInputParameters().then(async () => {
            const defaultDeviceName = Common_1.CommandLineUtils.platformFlagIsIOS(this.flags.platform)
                ? PlatformConfig_1.PlatformConfig.iOSConfig().defaultSimulatorName
                : PlatformConfig_1.PlatformConfig.androidConfig().defaultEmulatorName;
            this.deviceName = Common_1.CommandLineUtils.resolveFlag(this.flags.target, defaultDeviceName);
            this.componentName = Common_1.CommandLineUtils.resolveFlag(this.flags.componentname, '').trim();
            this.targetApp = Common_1.CommandLineUtils.resolveFlag(this.flags.targetapp, PreviewUtils_1.PreviewUtils.BROWSER_TARGET_APP);
            this.projectDir = CommonUtils_1.CommonUtils.resolveUserHomePath(Common_1.CommandLineUtils.resolveFlag(this.flags.projectdir, process.cwd()));
            const configFileName = CommonUtils_1.CommonUtils.resolveUserHomePath(Common_1.CommandLineUtils.resolveFlag(this.flags.configfile, ''));
            this.configFilePath = path_1.default.normalize(path_1.default.resolve(this.projectDir, configFileName));
            const hasConfigFile = this.configFilePath.length > 0 &&
                fs_1.default.existsSync(this.configFilePath);
            const isBrowserTargetApp = PreviewUtils_1.PreviewUtils.isTargetingBrowser(this.targetApp);
            this.logger.debug('Validating Preview command inputs.');
            if (isBrowserTargetApp && hasConfigFile) {
                this.logger.warn(messages.getMessage('ignoringConfigFileFlagDescription'));
            }
            if (this.componentName.length === 0) {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidComponentNameFlagsDescription'), 'lwc-dev-mobile', Preview.examples));
            }
            if (isBrowserTargetApp === false && hasConfigFile === false) {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:missingDescription', [this.configFilePath]), 'lwc-dev-mobile', Preview.examples));
            }
            if (isBrowserTargetApp === false && hasConfigFile === true) {
                const validationResult = await PreviewUtils_1.PreviewUtils.validateConfigFileWithSchema(this.configFilePath, configSchema);
                if (validationResult.passed === false) {
                    return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:genericDescription', [
                        this.configFilePath,
                        validationResult.errorMessage
                    ]), 'lwc-dev-mobile'));
                }
                const configFileContent = PreviewUtils_1.PreviewUtils.loadConfigFile(this.configFilePath);
                this.appConfig = configFileContent.getAppConfig(this.flags.platform, this.targetApp);
                if (this.appConfig === undefined) {
                    const errMsg = messages.getMessage('error:invalidConfigFile:missingAppConfigDescription', [this.targetApp, this.flags.platform]);
                    return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:genericDescription', [this.configFilePath, errMsg]), 'lwc-dev-mobile'));
                }
            }
            if (PreviewUtils_1.PreviewUtils.useLwcServerForPreviewing(this.targetApp, this.appConfig)) {
                const port = await CommonUtils_1.CommonUtils.getLwcServerPort();
                this.serverPort = port
                    ? port
                    : CommonUtils_1.CommonUtils.DEFAULT_LWC_SERVER_PORT;
            }
            return Promise.resolve();
        });
    }
    async launchPreview() {
        let appBundlePath;
        if (PreviewUtils_1.PreviewUtils.isTargetingBrowser(this.targetApp) === false &&
            this.appConfig) {
            try {
                CommonUtils_1.CommonUtils.startCliAction('Preview', 'Fetching app bundle path');
                appBundlePath = PreviewUtils_1.PreviewUtils.getAppBundlePath(path_1.default.dirname(this.configFilePath), this.appConfig);
            }
            catch (error) {
                CommonUtils_1.CommonUtils.stopCliAction('Error encountered while fetching app bundle path');
                return Promise.reject(error);
            }
        }
        if (Common_1.CommandLineUtils.platformFlagIsIOS(this.flags.platform)) {
            const config = this.appConfig && this.appConfig;
            return this.launchIOS(this.deviceName, this.componentName, this.projectDir, appBundlePath, this.targetApp, config, this.serverPort);
        }
        else {
            const config = this.appConfig && this.appConfig;
            return this.launchAndroid(this.deviceName, this.componentName, this.projectDir, appBundlePath, this.targetApp, config, this.serverPort);
        }
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:lwc:preview', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    _help() {
        const isCommandHelp = this.argv.filter((v) => v.toLowerCase() === '-h' || v.toLowerCase() === '--help').length > 0;
        if (isCommandHelp) {
            super._help();
        }
        else {
            const message = messages.getMessage('configFileHelpDescription');
            console.log(`${message}`);
        }
        return this.exit(0);
    }
    async launchIOS(deviceName, componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort) {
        const launcher = new IOSLauncher_1.IOSLauncher(deviceName);
        return launcher.launchPreview(componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort);
    }
    async launchAndroid(deviceName, componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort) {
        const launcher = new AndroidLauncher_1.AndroidLauncher(deviceName);
        return launcher.launchPreview(componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort);
    }
}
exports.Preview = Preview;
Preview.description = messages.getMessage('commandDescription');
Preview.args = [{ name: 'file' }];
Preview.flagsConfig = {
    componentname: command_1.flags.string({
        char: 'n',
        description: messages.getMessage('componentnameFlagDescription'),
        required: true
    }),
    configfile: command_1.flags.string({
        char: 'f',
        description: messages.getMessage('configFileFlagDescription'),
        required: false
    }),
    confighelp: command_1.flags.help({
        default: false,
        description: messages.getMessage('configHelpFlagDescription'),
        required: false
    }),
    platform: command_1.flags.string({
        char: 'p',
        description: messages.getMessage('platformFlagDescription'),
        required: true
    }),
    projectdir: command_1.flags.string({
        char: 'd',
        description: messages.getMessage('projectDirFlagDescription'),
        required: false
    }),
    target: command_1.flags.string({
        char: 't',
        description: messages.getMessage('targetFlagDescription'),
        required: false
    }),
    targetapp: command_1.flags.string({
        char: 'a',
        description: messages.getMessage('targetAppFlagDescription'),
        required: false
    })
};
Preview.requiresUsername = false;
Preview.supportsDevhubUsername = false;
Preview.requiresProject = false;
class LWCServerPluginInstalledRequirement {
    constructor(logger) {
        this.title = messages.getMessage('reqs:serverInstalled:title');
        this.fulfilledMessage = messages.getMessage('reqs:serverInstalled:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:serverInstalled:unfulfilledMessage');
        this.logger = logger;
    }
    async checkFunction() {
        return CommonUtils_1.CommonUtils.isLwcServerPluginInstalled()
            .then(() => {
            this.logger.info('sfdx server plugin detected.');
            return Promise.resolve(this.fulfilledMessage);
        })
            .catch((error) => {
            this.logger.info('sfdx server plugin was not detected.');
            try {
                const command = 'sfdx plugins:install @salesforce/lwc-dev-server';
                this.logger.info(`Installing sfdx server plugin.... ${command}`);
                CommonUtils_1.CommonUtils.executeCommandSync(command, [
                    'inherit',
                    'pipe',
                    'inherit'
                ]);
                this.logger.info('sfdx server plugin installed.');
                return Promise.resolve(this.fulfilledMessage);
            }
            catch (error) {
                this.logger.error(`sfdx server plugin installion failed. ${error}`);
                return Promise.reject(new Error(this.unfulfilledMessage));
            }
        });
    }
}
exports.LWCServerPluginInstalledRequirement = LWCServerPluginInstalledRequirement;
class LwcServerIsRunningRequirement {
    constructor(logger) {
        this.title = messages.getMessage('reqs:serverStarted:title');
        this.fulfilledMessage = messages.getMessage('reqs:serverStarted:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:serverStarted:unfulfilledMessage');
        this.logger = logger;
    }
    async checkFunction() {
        return CommonUtils_1.CommonUtils.getLwcServerPort().then((port) => {
            if (!port) {
                return Promise.reject(this.unfulfilledMessage);
            }
            else {
                return Promise.resolve(util_1.default.format(this.fulfilledMessage, port));
            }
        });
    }
}
exports.LwcServerIsRunningRequirement = LwcServerIsRunningRequirement;
//# sourceMappingURL=preview.js.map